@@ -6,8 +6,9 @@
  *
  * This is free software. See COPYING for details.
  */
-
 #if !NOGCONF
+using System.Runtime.InteropServices;
+
 namespace FSpot
 {
 	public class GConfPreferenceBackend : IPreferenceBackend
@@ -29,7 +30,7 @@ namespace FSpot
 		{
 			try {
 				return Client.Get (key);
-			} catch (GConf.NoSuchKeyException e) {
+			} catch (GConf.NoSuchKeyException) {
 				throw new NoSuchKeyException (key);
 			}
 		}
