@@ -438,7 +438,7 @@ namespace FSpot.Tiff {
 			Charset = charset;
 			// for (int i = 0; i < raw_data.Length; i++)
 			//	System.Console.WriteLine ("{0} - \"{1}\"", raw_data [i].ToString ("x"), raw_data [i]);
-			Value = enc.GetString (raw_data, 8, raw_data.Length - 8);
+			Value = enc.GetString (raw_data, 8, raw_data.Length - 8).Trim('\0');
 		}
 
 		public byte [] GetBytes (bool is_little)
@@ -459,7 +459,7 @@ namespace FSpot.Tiff {
 				heading = "ASCII\0\0\0";
 				enc = new System.Text.ASCIIEncoding ();
 			} else {
-				heading = "Unicode\0";
+				heading = "UNICODE\0";
 				enc = new System.Text.UnicodeEncoding (! is_little, true);
 			}
 			
@@ -950,15 +950,15 @@ namespace FSpot.Tiff {
 					break;
 				case TagId.ImageDescription:
 					MetadataStore.AddLiteral (sink, "dc:description", "rdf:Alt", 
-								  new Literal (e.ValueAsString [0], "x-default", null));
+								  new SemWeb.Literal (e.ValueAsString [0], "x-default", null));
 					break;
 				case TagId.UserComment:
 					MetadataStore.AddLiteral (sink, "exif:UserComment", "rdf:Alt", 
-								  new Literal (e.ValueAsString [0], "x-default", null));
+								  new SemWeb.Literal (e.ValueAsString [0], "x-default", null));
 					break;
 				case TagId.Copyright:
 					MetadataStore.AddLiteral (sink, "dc:rights", "rdf:Alt", 
-								  new Literal (e.ValueAsString [0], "x-default", null));
+								  new SemWeb.Literal (e.ValueAsString [0], "x-default", null));
 					break;
 				case TagId.Artist:
 					MetadataStore.Add (sink, "dc:creator", "rdf:Seq", e.ValueAsString);
@@ -1005,11 +1005,11 @@ namespace FSpot.Tiff {
 					
 					Statement cols = new Statement (empty, 
 									(Entity) MetadataStore.Namespaces.Resolve ("exif:Columns"),
-									new Literal (pattern.Columns.ToString (), null, null));
+									new SemWeb.Literal (pattern.Columns.ToString (), null, null));
 					sink.Add (cols);
 					Statement rows = new Statement (empty, 
 									(Entity) MetadataStore.Namespaces.Resolve ("exif:Rows"),
-									new Literal (pattern.Rows.ToString (), null, null));
+									new SemWeb.Literal (pattern.Rows.ToString (), null, null));
 					sink.Add (rows);
 					string [] vals = e.ArrayToString (pattern.Values);
 					MetadataStore.Add (sink, empty, "exif:Values", "rdf:Seq", vals);
@@ -1504,6 +1504,8 @@ namespace FSpot.Tiff {
 			for (int i = 0; i <  entry_count; i++) {
 				try {
 					directory_offset = BitConverter.ToUInt32 (raw_data, i * 4, endian == Endian.Little);
+					if (directory_offset == offset_origin + 8)
+						throw new Exception ("recursive ifd");
 					Directory [i] = new ImageDirectory (stream, directory_offset, endian);
 				} catch (System.Exception e) {
 					System.Console.WriteLine ("Error loading Subdirectory {0} at {2} of {3}bytes:{4}{1}", 
@@ -2285,6 +2287,14 @@ namespace FSpot.Tiff {
 
 		public Cr2File (Uri uri) : base (uri)
 		{
+//							Gtk.MessageDialog md = new Gtk.MessageDialog (null, 
+//							                                              Gtk.DialogFlags.DestroyWithParent,
+//							                                              Gtk.MessageType.Error, 
+//							                                              Gtk.ButtonsType.Close,
+//							                                              "bca");
+//	
+//							int result = md.Run ();
+//							md.Destroy();
 		}
 
 		/*
