@@ -359,7 +359,11 @@ namespace FSpot {
 			int cy = Center.Y;
 		
 			g.Operator = Operator.Source;
+#if MONO_1_2_5
 			g.Source = new SolidPattern (new Cairo.Color (0,0,0,0));
+#else
+			g.Source = new SolidPattern (new Cairo.Color (0,0,0,0), true);
+#endif
 			g.Rectangle (0, 0, width, height);
 			g.Paint ();
 
@@ -367,14 +371,22 @@ namespace FSpot {
 			g.Translate (cx, cy);
 			g.Rotate (angle);
 
+#if MONO_1_2_5
 			g.Source = new SolidPattern (new Cairo.Color (0.2, 0.2, 0.2, .6));
+#else
+			g.Source = new SolidPattern (new Cairo.Color (0.2, 0.2, 0.2, .6), true);
+#endif
 			g.Operator = Operator.Over;
 			g.Rectangle (0, - (border + inner), inner_x, 2 * (border + inner));
 			g.Arc (inner_x, 0, inner + border, 0, 2 * Math.PI);
 			g.Arc (0, 0, radius + border, 0, 2 * Math.PI);
 			g.Fill ();
 
+#if MONO_1_2_5
 			g.Source = new SolidPattern (new Cairo.Color (0, 0, 0, 1.0));
+#else
+			g.Source = new SolidPattern (new Cairo.Color (0, 0, 0, 1.0), true);
+#endif
 			g.Operator = Operator.DestOut;
 			g.Arc (inner_x, 0, inner, 0, 2 * Math.PI);
 #if true			
@@ -414,7 +426,11 @@ namespace FSpot {
 				g.Arc (0, 0, radius, angle, angle + Math.PI);
 				g.ClosePath ();
 				g.FillPreserve ();
+#if MONO_1_2_5
 				g.Source = new SolidPattern (new Cairo.Color (1.0, 1.0, 1.0, 1.0));
+#else
+				g.Source = new SolidPattern (new Cairo.Color (1.0, 1.0, 1.0, 1.0), true);
+#endif
 				g.Stroke ();
 			}
 		}
@@ -448,13 +464,13 @@ namespace FSpot {
 
 		}
 		
-                [DllImport ("libcairo-2.dll")]
-                static extern void cairo_user_to_device (IntPtr cr, ref double x, ref double y);
-
-		static void UserToDevice (Context ctx, ref double x, ref double y)
-		{
-			cairo_user_to_device (ctx.Handle, ref x, ref y);
-		}
+//                [DllImport ("libcairo-2.dll")]
+//                static extern void cairo_user_to_device (IntPtr cr, ref double x, ref double y);
+//
+//		private static void UserToDevice (Context ctx, ref double x, ref double y)
+//		{
+//			cairo_user_to_device (ctx.Handle, ref x, ref y);
+//		}
 
 		
 		bool dragging = false;
@@ -561,7 +577,7 @@ namespace FSpot {
 				break;
 			case Gdk.EventType.TwoButtonPress:
 				dragging = false;
-				this.Destroy ();
+				MainWindow.Toplevel.HideLoupe ();
 				break;
 			}
 		}
@@ -580,7 +596,7 @@ namespace FSpot {
 		{
 			switch (args.Event.Key) {
 			case Gdk.Key.v:
-				Destroy ();
+				MainWindow.Toplevel.HideLoupe ();
 				args.RetVal = true;
 				break;
 			default:
