@@ -1,5 +1,6 @@
 using System;
 using FSpot.Editors;
+using FSpot.Utils;
 
 namespace FSpot {
 	public enum ProgressType {
@@ -18,16 +19,25 @@ namespace FSpot {
 		ProgressType load_async = ProgressType.Full;
 		bool progressive_display;
 		public GdkGlx.Context Glx;
-		private Editor editor;
+		private OldEditor editor;
 
-		public PhotoImageView (IBrowsableCollection query)
+		public PhotoImageView (IBrowsableCollection query) : this (new BrowsablePointer (query, -1))
+		{
+			FSpot.ColorManagement.PhotoImageView = this;
+		}
+
+		public PhotoImageView (BrowsablePointer item)
 		{
 			loader = new FSpot.AsyncPixbufLoader ();
 			loader.AreaUpdated += HandlePixbufAreaUpdated;
 			loader.AreaPrepared += HandlePixbufPrepared;
 			loader.Done += HandleDone;
+			
+			FSpot.ColorManagement.PhotoImageView = this;
+			this.Transform = FSpot.ColorManagement.StandartTransform (); //for preview windows
 
 			Accelerometer.OrientationChanged += HandleOrientationChanged;
+
 			HandleRealized (null, null);
 
 			this.SizeAllocated += HandleSizeAllocated;
@@ -35,7 +45,7 @@ namespace FSpot {
 			//this.Realized += HandleRealized;
 			this.Unrealized += HandleUnrealized;
 			this.ScrollEvent += HandleScrollEvent;
-			this.item = new BrowsablePointer (query, -1);
+			this.item = item;
 			item.Changed += PhotoItemChanged;
 			this.Destroyed += HandleDestroyed;
 			this.SetTransparentColor (this.Style.BaseColors [(int)Gtk.StateType.Normal]);
@@ -71,19 +81,32 @@ namespace FSpot {
 #endif
 		}
 
-		public Editor Editor {
+		public OldEditor Editor {
 			get { return editor; }
 			set {
-				Editor tmp = editor;
+				value.Done += HandleEditorDone;
 
-				editor = value;
-
-				if (tmp != null)
-					tmp.Destroy ();
+				if (editor != null)
+					editor.Destroy ();
 				
+				editor = value;
 			}
 		}
 
+		private void HandleEditorDone (object sender, EventArgs args)
+		{
+			OldEditor old = sender as OldEditor;
+
+			old.Done -= HandleEditorDone;
+				
+			if (old == editor)
+				editor = null;
+		}
+
+		public Loupe Loupe {
+			get { return loupe; }
+		}
+
 		public Gdk.Pixbuf CompletePixbuf ()
 		{
 			loader.LoadToDone ();
@@ -97,7 +120,7 @@ namespace FSpot {
 
 		public void Reload ()
 		{
-			if (!Item.IsValid)
+			if (Item == null || !Item.IsValid)
 				return;
 			
 			PhotoItemChanged (Item, null);
@@ -113,13 +136,18 @@ namespace FSpot {
 				(int) GdkGlx.GlxAttribute.None
 			};
 
-			Glx = new GdkGlx.Context (Screen, attr);
-		        Colormap = Glx.GetColormap ();
+			try {
+				Glx = new GdkGlx.Context (Screen, attr);
+				Colormap = Glx.GetColormap ();
+			} catch (GdkGlx.GlxException e) {
+				Console.WriteLine ("Error initializing the OpenGL context:{1} {0}", e, Environment.NewLine);
+			}
 		}
 
 		private void HandleUnrealized (object sender, EventArgs args)
 		{
-			Glx.Destroy ();
+			if (Glx != null)
+				Glx.Destroy ();
 		}
 
 		// Display.
@@ -281,7 +309,6 @@ namespace FSpot {
 
 		private void PhotoItemChanged (BrowsablePointer item, BrowsablePointerChangedArgs args) 
 		{
-			System.Console.WriteLine ("item changed");
 			// If it is just the position that changed fall out
 			if (args != null && 
 			    args.PreviousItem != null &&
@@ -290,6 +317,14 @@ namespace FSpot {
 			    (this.Item.Current.DefaultVersionUri == args.PreviousItem.DefaultVersionUri))
 				return;
 
+			// Don't reload if the image didn't change at all.
+			if (args != null && args.Changes != null &&
+			    !args.Changes.DataChanged &&
+			    args.PreviousItem != null &&
+			    Item.IsValid &&
+			    this.Item.Current.DefaultVersionUri == args.PreviousItem.DefaultVersionUri)
+				return;
+
 			if (args != null &&
 			    args.PreviousItem != null && 
 			    Item.IsValid && 
@@ -345,7 +380,7 @@ namespace FSpot {
 			ZoomFit (upscale);
 		}
 
-		private void ZoomFit (bool upscale)
+		public void ZoomFit (bool upscale)
 		{			
 			Gdk.Pixbuf pixbuf = this.Pixbuf;
 			Gtk.ScrolledWindow scrolled = this.Parent as Gtk.ScrolledWindow;
@@ -354,24 +389,19 @@ namespace FSpot {
 			if (pixbuf == null)
 				return;
 
-			int available_width = this.Allocation.Width;
-			int available_height = this.Allocation.Height;
-		
+			if (scrolled != null)
+				scrolled.SetPolicy (Gtk.PolicyType.Never, Gtk.PolicyType.Never);
+
+			int available_width = (scrolled != null) ? scrolled.Allocation.Width : this.Allocation.Width;
+			int available_height = (scrolled != null) ? scrolled.Allocation.Height : this.Allocation.Height;
+
 			double zoom_to_fit = ZoomUtils.FitToScale ((uint) available_width, 
 								   (uint) available_height,
 								   (uint) pixbuf.Width, 
 								   (uint) pixbuf.Height, 
 								   upscale);
-			
-			double image_zoom = zoom_to_fit;
-			/*
-			System.Console.WriteLine ("Zoom = {0}, {1}, {2}", image_zoom, 
-						  available_width, 
-						  available_height);
-			*/
 
-			if (scrolled != null)
-				scrolled.SetPolicy (Gtk.PolicyType.Never, Gtk.PolicyType.Never);
+			double image_zoom = zoom_to_fit;
 
 			this.SetZoom (image_zoom, image_zoom);
 			
@@ -392,34 +422,84 @@ namespace FSpot {
 		[GLib.ConnectBefore]
 		private void HandleKeyPressEvent (object sender, Gtk.KeyPressEventArgs args)
 		{
+			bool alt = Gdk.ModifierType.Mod1Mask == (args.Event.State & Gdk.ModifierType.Mod1Mask);
+
 			// FIXME I really need to figure out why overriding is not working
 			// for any of the default handlers.
 			args.RetVal = true;
 		
-			// Check for KeyPad arrow keys, which scroll the window when zoomed in
-			// but should go to the next/previous photo when not zoomed (no scrollbars)
-			if (this.Fit) {
-				switch (args.Event.Key) {
+			// Scroll if image is zoomed in (scrollbars are visible)
+			Gtk.ScrolledWindow scrolled = this.Parent as Gtk.ScrolledWindow;
+			if (scrolled != null && !this.Fit) {
+				Gtk.Adjustment vadj = scrolled.Vadjustment;
+				Gtk.Adjustment hadj = scrolled.Hadjustment;
+				switch (args.Event.Key) {					
 				case Gdk.Key.Up:
-				case Gdk.Key.Left:
 				case Gdk.Key.KP_Up:
+				case Gdk.Key.k:
+				case Gdk.Key.K:
+					vadj.Value -= vadj.StepIncrement;
+					if (vadj.Value < vadj.Lower)
+						vadj.Value = vadj.Lower;
+					return;
+				case Gdk.Key.Left:
 				case Gdk.Key.KP_Left:
-					this.Item.MovePrevious ();
+				case Gdk.Key.h:
+					hadj.Value -= hadj.StepIncrement;
+					if (hadj.Value < hadj.Lower)
+						hadj.Value = hadj.Lower;
 					return;
 				case Gdk.Key.Down:
-				case Gdk.Key.Right:
 				case Gdk.Key.KP_Down:
+				case Gdk.Key.j:
+				case Gdk.Key.J:
+					vadj.Value += vadj.StepIncrement;
+					if (vadj.Value > vadj.Upper - vadj.PageSize)
+						vadj.Value = vadj.Upper - vadj.PageSize;
+					return;
+				case Gdk.Key.Right:
 				case Gdk.Key.KP_Right:
-					this.Item.MoveNext ();
+				case Gdk.Key.l:
+					hadj.Value += hadj.StepIncrement;
+					if (hadj.Value > hadj.Upper - hadj.PageSize)
+						hadj.Value = hadj.Upper - hadj.PageSize;
 					return;
 				}
 			}
-
+			
+			// Go to the next/previous photo when not zoomed (no scrollbars)
 			switch (args.Event.Key) {
+			case Gdk.Key.Up:
+			case Gdk.Key.KP_Up:
+			case Gdk.Key.Left:
+			case Gdk.Key.KP_Left:
 			case Gdk.Key.Page_Up:
 			case Gdk.Key.KP_Page_Up:
+			case Gdk.Key.BackSpace:
+			case Gdk.Key.h:
+			case Gdk.Key.H:
+			case Gdk.Key.k:
+			case Gdk.Key.K:
+			case Gdk.Key.b:
+			case Gdk.Key.B:
 				this.Item.MovePrevious ();
 				break;
+			case Gdk.Key.Down:
+			case Gdk.Key.KP_Down:
+			case Gdk.Key.Right:
+			case Gdk.Key.KP_Right:
+			case Gdk.Key.Page_Down:
+			case Gdk.Key.KP_Page_Down:
+			case Gdk.Key.space:
+			case Gdk.Key.KP_Space:
+			case Gdk.Key.j:
+			case Gdk.Key.J:
+			case Gdk.Key.l:
+			case Gdk.Key.L:
+			case Gdk.Key.n:
+			case Gdk.Key.N:
+				this.Item.MoveNext ();
+				break;
 			case Gdk.Key.Home:
 			case Gdk.Key.KP_Home:
 				this.Item.Index = 0;
@@ -428,46 +508,31 @@ namespace FSpot {
 			case Gdk.Key.KP_End:
 				this.Item.Index = this.Query.Count - 1;
 				break;
-			case Gdk.Key.Page_Down:
-			case Gdk.Key.KP_Page_Down:
-			case Gdk.Key.space:
-			case Gdk.Key.KP_Space:
-				this.Item.MoveNext ();
-				break;
 			case Gdk.Key.Key_0:
 			case Gdk.Key.KP_0:
-				this.Fit = true;
+				if (alt) 
+					args.RetVal = false;
+				else
+					this.Fit = true;
 				break;
 			case Gdk.Key.Key_1:
 			case Gdk.Key.KP_1:
-				this.Zoom =  1.0;
+				if (alt)
+					args.RetVal = false;
+				else
+					this.Zoom =  1.0;
 				break;
 			case Gdk.Key.Key_2:
 			case Gdk.Key.KP_2:
-				this.Zoom = 2.0;
+				if (alt) 
+					args.RetVal = false;
+				else
+					this.Zoom = 2.0;
 				break;
 			case Gdk.Key.minus:
 			case Gdk.Key.KP_Subtract:
 				ZoomOut ();
 				break;
-			case Gdk.Key.s:
-				if (sharpener == null) {
-					sharpener = new Sharpener (this);
-					sharpener.Destroyed += HandleLoupeDestroy;
-				}
-
-				sharpener.Show ();
-				break;
-			case Gdk.Key.v:
-				if (loupe == null) {
-					loupe = new Loupe (this);
-					loupe.Destroyed += HandleLoupeDestroy;
-				}
-				loupe.Show ();
-				break;
-			case Gdk.Key.e:
-				Editor = new FSpot.Editors.SoftFocus (this);
-				break;
 			case Gdk.Key.equal:
 			case Gdk.Key.plus:
 			case Gdk.Key.KP_Add:
@@ -480,6 +545,28 @@ namespace FSpot {
 
 			return;
 		}
+
+		public void ShowHideLoupe ()
+		{
+			if (loupe == null) {
+				loupe = new Loupe (this);
+				loupe.Destroyed += HandleLoupeDestroy;
+				loupe.Show ();
+			} else {
+				loupe.Destroy ();	
+			}
+			
+		}
+		
+		public void ShowSharpener ()
+		{
+			if (sharpener == null) {
+				sharpener = new Sharpener (this);
+				sharpener.Destroyed += HandleLoupeDestroy;
+			}
+
+			sharpener.Show ();	
+		}
 		
 		[GLib.ConnectBefore]
 		private void HandleScrollEvent (object sender, Gtk.ScrollEventArgs args)
