@@ -1,7 +1,7 @@
 using Gtk;
 using GtkSharp;
 using System;
-
+using FSpot;
 public class PhotoVersionMenu : Menu {
 	private uint version_id;
 	public uint VersionId {
@@ -52,9 +52,11 @@ public class PhotoVersionMenu : Menu {
 
 		int i = 0;
 		foreach (uint id in version_ids) {
-			MenuItem menu_item = new MenuItem (photo.GetVersionName (id));
+			MenuItem menu_item = new MenuItem (photo.GetVersion (id).Name);
 			menu_item.Show ();
 			menu_item.Sensitive = true;
+			((Gtk.Label)menu_item.Child).UseUnderline = false;
+			((Gtk.Label)menu_item.Child).Ellipsize = Pango.EllipsizeMode.End;
 			menu_item.Activated += new EventHandler (HandleMenuItemActivated);
 
 			item_infos [i ++] = new MenuItemInfo (menu_item, id);
@@ -63,9 +65,10 @@ public class PhotoVersionMenu : Menu {
 		}
 
 		if (version_ids.Length == 1) {
-			MenuItem no_edits_menu_item = new MenuItem (Mono.Posix.Catalog.GetString ("(No Edits)"));
+			MenuItem no_edits_menu_item = new MenuItem (Mono.Unix.Catalog.GetString ("(No Edits)"));
 			no_edits_menu_item.Show ();
 			no_edits_menu_item.Sensitive = false;
+			((Gtk.Label)no_edits_menu_item.Child).Ellipsize = Pango.EllipsizeMode.End;
 			Append (no_edits_menu_item);
 		}
 	}
